// Copyright (c) 1997 Andrew Maltsev <am@amsoft.ru>
// All rights reserved
// 
// Miscellaneous methods - memmory allocating, printing etc..
//
#include <stdlib.h>
#include <unistd.h>
#include "es_int.h"

///////////////////////////////////////////////////////////////////////////////
// Formatted printing into dynamycally allocated buffer. Buffer have to
// be free'd when not needed any more..
//
char *
vaprintf(char const *fmt, va_list ap)
{ if(!fmt) return NULL;
  char *s=NULL;
  for(int sz=400; 1; sz+=400)
   { s=(typeof(s))reAlloc(s,sizeof(*s)*sz);
     if(vsnprintf(s,sz,fmt,ap)<sz)
      break;
     log(D_MISC,"vaprintf: sz=%d, fmt=`%s'",sz,fmt);
   }
  return s;
}

///////////////////////////////////////////////////////////////////////////////
// Formatted printing into internal buffer of arbitrary size
// Suitable in cases like `f=fopen(qprintf("%s/%s",dir,path),"r");'
//
char const *
qprintf(char const *fmt, ...)
{ if(!fmt) return NULL;
  static char *str=NULL;
  static unsigned sz=0;
  va_list ap;
  va_start(ap,fmt);
  if(sz<strlen(fmt)+100)
   str=(typeof(str))reAlloc(str,sizeof(*str)*(sz=strlen(fmt)+100));
  while(vsnprintf(str,sz,fmt,ap)>=(int)sz)
   str=(typeof(str))reAlloc(str,sizeof(*str)*(sz+=500));
  va_end(ap);
  return str;
} 

///////////////////////////////////////////////////////////////////////////////
// Safe memory allocating -- waits until success.
// `Three in one'.
//
void *
reAlloc(void *mem, unsigned long size)
{ if(!size)
   { if(mem) free(mem);
     return NULL;
   }
  void *r;
  while((r=realloc(mem,size))==NULL)
   { log(D_ERROR|D_MISC,"No memory!");
     sleep(60);
   }
  return r;
}

void *
mAlloc(unsigned long size)
{ return reAlloc(NULL,size);
}

void
Free(void *mem)
{ reAlloc(mem,0);
}

///////////////////////////////////////////////////////////////////////////////
// Safe string duplicating
//
char *
strDup(char const *org)
{ if(!org) return NULL;
  char *s=(char *)mAlloc(sizeof(char)*(strlen(org)+1));
  return strcpy(s,org);
}

///////////////////////////////////////////////////////////////////////////////
R("$Id: misc.C,v 1.6 1997/10/23 18:40:42 am Exp $");
